﻿using System.Collections;
using System.IO.Ports;
using System.Threading;
using UnityEngine;

/// <summary>
/// Class for serial communication
/// </summary>
public class SerialHandler : MonoBehaviour
{
  /// <summary>
  ///     delegate type that receives one argument of type string
  /// </summary>
  /// <param name="message"></param>
  public delegate void SerialReadEventHandler(string message);

  /// <summary>
  ///     Define the delegate type event created above
  /// </summary>
  public event SerialReadEventHandler OnDataReceived = delegate { };

  /// <summary>
  /// port name used for serial communication
  /// </summary>
  public string portName = "/dev/tty.usbmodem1421 or COM1";

  /// <summary>
  /// Match with Arduino settings
  /// </summary>
  public int baudRate = 19200;

  /// <summary>
  /// SerialPort
  /// </summary>
  private SerialPort port;

  /// <summary>
  /// readingData
  /// </summary>
  private string readData;

  /// <summary>
  ///The method called at startup 
  /// </summary>
  void Awake()
  {
    port = new SerialPort(portName, baudRate);
    port.ReadTimeout = 1000;
    port.NewLine = "\n";
    port.Open();

    StartCoroutine("Read");
  }

  void Start()
  {
    
  }

  /// <summary>
  /// Method called every frame
  /// </summary>
  void Update()
  {
    if (readData.Length > 0)
    {
      OnDataReceived(readData);
      readData = "";
    }
  }

  /// <summary>
  /// Method called when destroying instance
  /// </summary>
  void OnDestroy()
  {
    if (port != null && port.IsOpen)
    {
      port.Close();
      port.Dispose();
    }
  }

  /// <summary>
  /// Read from the port each 100ms
  /// </summary>
  private IEnumerator Read()
  {
    while (port != null && port.IsOpen)
    {
      try
      {
        readData = port.ReadLine();
      }
      catch (System.Exception e)
      {
        Debug.LogWarning(e.Message);
      }
      yield return new WaitForSeconds(0.1f);
    }
  }

  /// <summary>
  /// Write to the port 
  /// </summary>
  /// <param name="message"></param>
  public void Write(string message)
  {
    try
    {
      port.Write(message);
    }
    catch (System.Exception e)
    {
      Debug.LogWarning(e.Message);
    }
  }
}